@@ -162,16 +162,18 @@ def fusion_cell_type_predictions_by_type(cell_type_preds_dict, default_cell_type
         print(" ")
         print(f"Number of cells for each cell type after fusion:")
         for code in np.arange(len(cell_type_from_code_dict)):
-            print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
+            if code >= len(cell_count_by_type):
+                print(f"0 {cell_type_from_code_dict[code]}")
+            else:
+                print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
         print(" ")
 
     return fusion_predictions
 
 
-
-
-def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
-                                 cell_type_config_file=None, filename_to_save=None):
+def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2, with_new_category=None,
+                                 cell_type_config_file=None, filename_to_save=None,
+                                 cell_types_to_not_fusion_with_gt=None):
     """
     Take two cell type predictions (should be for the same movie, so the same number of cells in both),
     and fusion it so that cells that have a cell type probabilities in cell_type_pred_2 (sum of probablities equals to 1)
@@ -187,10 +189,19 @@ def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
         cell_type_config_file: (str) yaml cell type that associate a cell type name to a code (one of the column
         in cell_type_pred)
         filename_to_save: (str) should have npy extension or it will be added, should include the full path.
+        cell_types_to_not_fusion_with_gt: list of int, representing the cell types that should be replaced by the
+        GT. For example if a cell is classified as Noise in pred_1 but INs in pred_2, then it stayed noise.
+        In that case cell_types_to_not_fusion_with_gt=[2]
+        with_new_category: list of tuple of 2 ints, if not None, then change the category in cell_type_pred_2 that
+        are given to cell_type_pred_1, by either swaping the 2 numbers, or creating a new one if non existent. The first
+        number category will then be set to zero and the 2nd one will take the probability of the first one
 
     Returns: a 2d array (n_cells, n_classes) representing the new cell type predictions will be returned
 
     """
+    if cell_types_to_not_fusion_with_gt is None:
+        cell_types_to_not_fusion_with_gt = []
+
     # first we check the that both cell pred have the same dimensions
     if cell_type_pred_1.shape[0] != cell_type_pred_2.shape[0]:
         print(f"Both cell type pred 1 & 2 should have the same number of cells instead we have "
@@ -205,6 +216,23 @@ def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
     # not to modify the original one
     cell_type_pred_1 = np.copy(cell_type_pred_1)
 
+    if with_new_category is not None:
+        # we check if a new category has an index superior to the limit
+        # if so we add one or more category
+        max_n_pred = cell_type_pred_1.shape[1] - 1
+        max_new_n_preds = 0
+        for cat_tuple in with_new_category:
+            if np.max(cat_tuple) > max_new_n_preds:
+                max_new_n_preds = np.max(cat_tuple)
+        if max_new_n_preds > max_n_pred:
+            # then we increase the size of arrays of both pred
+            new_cell_type_pred_1 = np.zeros((cell_type_pred_1.shape[0], max_new_n_preds+1))
+            new_cell_type_pred_1[:, :-1] = cell_type_pred_1
+            cell_type_pred_1 = new_cell_type_pred_1
+            new_cell_type_pred_2 = np.zeros((cell_type_pred_2.shape[0], max_new_n_preds + 1))
+            new_cell_type_pred_2[:, :-1] = cell_type_pred_2
+            cell_type_pred_2 = new_cell_type_pred_2
+
     if cell_type_config_file is not None:
         cell_type_from_code_dict, cell_type_to_code_dict, multi_class_arg = \
             read_cell_type_categories_yaml_file(yaml_file=cell_type_config_file)
@@ -224,7 +252,10 @@ def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
         print(" ")
         print(f"Number of cells for each cell type before fusion:")
         for code in np.arange(len(cell_type_from_code_dict)):
-            print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
+            if code >= len(cell_count_by_type):
+                print(f"0 {cell_type_from_code_dict[code]}")
+            else:
+                print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
         print(" ")
 
     n_cells_changed = 0
@@ -239,17 +270,43 @@ def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
 
         if cell_type_from_code_dict is not None:
             # we print some information
-            print(f"Cell type pred of cell {cell} were: {np.round(cell_type_pred_1[cell], 2)} ->"
+            print(f"Cell type pred of cell {cell} were: {np.round(cell_type_pred_1[cell], 2)} -> "
                   f"{cell_type_from_code_dict[np.argmax(cell_type_pred_1[cell])]}")
 
-        if np.argmax(cell_type_pred_1[cell]) != np.argmax(cell_type_pred_2[cell]):
-            n_cells_different += 1
-
-        cell_type_pred_1[cell] = cell_type_pred_2[cell]
-        if cell_type_from_code_dict is not None:
-            print(f"New ones are: {np.round(cell_type_pred_1[cell], 2)} ->"
+        if np.argmax(cell_type_pred_1[cell]) in cell_types_to_not_fusion_with_gt:
+            print(f"New ones are: {np.round(cell_type_pred_1[cell], 2)} -> "
                   f"{cell_type_from_code_dict[np.argmax(cell_type_pred_1[cell])]}")
             print(" ")
+        else:
+            if np.argmax(cell_type_pred_1[cell]) != np.argmax(cell_type_pred_2[cell]):
+                n_cells_different += 1
+            cell_prob_to_merge = cell_type_pred_2[cell]
+            if with_new_category is not None:
+                for categories_to_swap in with_new_category:
+                    first_cat = categories_to_swap[0]
+                    second_cat = categories_to_swap[1]
+                    if second_cat <= len(cell_prob_to_merge) - 1:
+                        first_prob = cell_prob_to_merge[first_cat]
+                        cell_prob_to_merge[first_cat] = cell_prob_to_merge[second_cat]
+                        cell_prob_to_merge[second_cat] = first_prob
+                    else:
+                        if second_cat > len(cell_prob_to_merge):
+                            raise Exception(f"New category should be next in the range of indices. We have {second_cat} "
+                                            f"for n indices to {len(cell_prob_to_merge)}")
+                        new_cell_prob = []
+                        for index in range(second_cat+1):
+                            if index == second_cat:
+                                new_cell_prob.append(cell_prob_to_merge[first_cat])
+                            elif index == first_cat:
+                                new_cell_prob.append(0)
+                            else:
+                                new_cell_prob.append(cell_prob_to_merge[index])
+                        cell_prob_to_merge = new_cell_prob
+            cell_type_pred_1[cell] = cell_prob_to_merge
+            if cell_type_from_code_dict is not None:
+                print(f"New ones are: {np.round(cell_type_pred_1[cell], 2)} ->"
+                      f"{cell_type_from_code_dict[np.argmax(cell_type_pred_1[cell])]}")
+                print(" ")
 
     if cell_type_from_code_dict is not None:
         print(f"{n_cells_changed} cells had their predictions changed.")
@@ -268,7 +325,10 @@ def fusion_cell_type_predictions(cell_type_pred_1, cell_type_pred_2,
         print(" ")
         print(f"Number of cells for each cell type after fusion:")
         for code in np.arange(len(cell_type_from_code_dict)):
-            print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
+            if code >= len(cell_count_by_type):
+                print(f"0 {cell_type_from_code_dict[code]}")
+            else:
+                print(f"{cell_count_by_type[code]} {cell_type_from_code_dict[code]}")
         print(" ")
 
     if filename_to_save is not None:
@@ -344,7 +404,7 @@ def select_activity_classifier_on_cell_type_outputs(cell_type_config_file, cell_
                     model_files_dict[default_classifier].append(cell)
                 else:
                     raise Exception(f"Cell type code {cell_type_code} not recognized as a valid cell type, "
-                                f"using {os.path.basename(cell_type_config_file)}")
+                                    f"using {os.path.basename(cell_type_config_file)}")
             else:
                 model_tuple = cell_type_to_classifier_encoded[cell_type_code]
                 if model_tuple not in model_files_dict:
@@ -1130,7 +1190,7 @@ def activity_predictions_from_cinac_files(cinac_dir_name, results_path,
             # predictions for the first cell
             # prediction_valuesis a 1d array of len n_frames in the segment
             prediction_values = predictions_for_cinac_file[0]
-            predictions_for_all_cells[cell, first_frame:last_frame+1] = prediction_values
+            predictions_for_all_cells[cell, first_frame:last_frame + 1] = prediction_values
             # we build the raster, putting 1 at every frame in which the cell is predicted as active
             # predicted_raster_dur = np.zeros(len(prediction_values), dtype="int8")
             # predicted_raster_dur[prediction_values > 0.5] = 1
